/* Copyright (c) 2010 Ondrej Donek.
 * See the file LICENSE.txt for licensing information. */

// Namespace
if(typeof(xpcomviewer) == "undefined") var xpcomviewer = {};
if(typeof(xpcomviewer.global) == "undefined") xpcomviewer.global = {};
// Implementation
(function() {
	
	/** @type Components.interfaces.nsIPrefBranch */
	const kPREF_SRV = Components.classes["@mozilla.org/preferences-service;1"].
			getService(Components.interfaces.nsIPrefService).
			getBranch("extensions.xpcomviewer.");
	
	/** @type Components.interfaces.nsIPromptService */
	const kPROMPT_SRV = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
			getService(Components.interfaces.nsIPromptService);
	
	/** @type Components.interfaces.mozIStorageService */
	const kSTORAGE_SRV = Components.classes["@mozilla.org/storage/service;1"].
			getService(Components.interfaces.mozIStorageService);
	
	/**
	 * Holds connection to XPCOMViewer's database.
	 * @private
	 * @type Components.interfaces.mozIStorageConnection
	 */
	var _DB = null;
	
	
	/**
	 * Returns database connection.
	 * 
	 * @private
	 * @returns {Components.interfaces.mozIStorageConnection}
	 */
	this.getDb = function() 
	{
		if(_DB == null) {
			let dbFile = Components.classes["@mozilla.org/file/directory_service;1"].
					getService(Components.interfaces.nsIProperties).
					get("ProfD", Components.interfaces.nsIFile);
			dbFile.append("xpcomviewer.sqlite");
			
			_DB = kSTORAGE_SRV.openDatabase(dbFile);
		}
		
		return _DB;
	}; // end _initDb()
	
	
	/**
	 * Copy given text to the clipboard.
	 * 
	 * @param aText {string}
	 * @returns {void}
	 */
	this.copyToClipboard = function(aText)
	{
		let text = new String(aText);
		let clip = Components.classes["@mozilla.org/widget/clipboard;1"].
				createInstance(Components.interfaces.nsIClipboard);
		let trans = Components.classes["@mozilla.org/widget/transferable;1"].
				createInstance(Components.interfaces.nsITransferable);
		let str = Components.classes["@mozilla.org/supports-string;1"].
				createInstance(Components.interfaces.nsISupportsString);
		str.data = text;

		trans.addDataFlavor("text/unicode");
		trans.setTransferData("text/unicode", str, text.length * 2);
		clip.setData(trans, null, Components.interfaces.nsIClipboard.kGlobalClipboard);
	}; // end copyToClipboard(aText)
	
	/** 
	 * Select file using filepicker.
	 * 
	 * @param aTitle {string}
	 * @returns {Components.interfaces.nsILocalFile}
	 */
	this.pickFile = function(aTitle) 
	{
		let fp = Components.classes["@mozilla.org/filepicker;1"].
				createInstance(Components.interfaces.nsIFilePicker);
		fp.init(window, aTitle, Components.interfaces.nsIFilePicker.modeSave);
		fp.appendFilters(0);

		let fpStatus = fp.show();
		if((fpStatus == Components.interfaces.nsIFilePicker.returnOK) ||
		   (fpStatus == Components.interfaces.nsIFilePicker.returnReplace)) {
			return fp.file;
		}
		
		return null;
	}; // end pickFile(aTitle) 
	

	/**
	 * Write given text document to the file.
	 * 
	 * @param aFile {Components.interfaces.nsILocalFile}
	 * @param aText {String}
	 * @returns {void}
	 */
	this.writeTextFile = function(aFile, aText) 
	{
		let foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
				createInstance(Components.interfaces.nsIFileOutputStream);
		
		//aFile.QueryInterface(Components.interfaces.nsILocalFile)
		if(!aFile.exists()) aFile.create(0x00, 0666);
		
		foStream.init(aFile, 0x02 | 0x08 | 0x20, 0666, 0);
		
		let converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
				createInstance(Components.interfaces.nsIConverterOutputStream);
		converter.init(foStream, "UTF-8", 0, 0x0000);
		converter.writeString(aText);
		converter.close();
	}; // end writeTextFile(aFile, aText)
	

	/**
	 * Write given XML document to the file.
	 * 
	 * @param aFile {Components.interfaces.nsILocalFile}
	 * @param aDocument {Components.interfaces.nsIDOMDocument}
	 * @returns {void}
	 */
	this.writeXmlFile = function(aFile, aDocument) 
	{
		let foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
				createInstance(Components.interfaces.nsIFileOutputStream);
		
		//aFile.QueryInterface(Components.interfaces.nsILocalFile)
		if(!aFile.exists()) aFile.create(0x00, 0666);
		
		foStream.init(aFile, 0x02 | 0x08 | 0x20, 0666, 0);
		
		let serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].
				createInstance(Components.interfaces.nsIDOMSerializer);
		serializer.serializeToStream(aDocument, foStream, "");
	}; // end writeXmlFile(aFile, aDocument)
	
	
	/**
	 * Shows alert dialog with the checkbox "Show this dialog again"
	 * 
	 * @param aMessage {String}
	 * @param aBoolPrefId {String}
	 * @returns {void}
	 */
	this.showAlert = function(aMessage, aBoolPrefId)
	{
		let strBundle = document.getElementById("stringbundle");
		let showDlg = true;
		
		if(kPREF_SRV.prefHasUserValue(aBoolPrefId)) {
			showDlg = kPREF_SRV.getBoolPref(aBoolPrefId);
		}
		
		if(showDlg) {
			let check = { value: false }; 
			kPROMPT_SRV.alertCheck(
					window, 
					strBundle.getString("alertDlgTitle"), 
					strBundle.getString(aMessage),
					strBundle.getString("alertCheckMsg"), 
					check
			);
			kPREF_SRV.setBoolPref(aBoolPrefId, !check.value);
		}
	}; // end showAlert(aMessage, aBoolPrefId)
	
	
	/**
	 * Returns encoded URL.
	 * 
	 * @author https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/encodeURI
	 * @returns {String}
	 */
	this.encodeUri = function(aStr) {  
		return encodeURI(aStr).replace(/%5B/g, '[').replace(/%5D/g, ']');  
	}; // end encodeUri(aStr)
	
	
	/**
	 * Opens given URL. On Firefox, Thunderbird or SeaMonkey opens given URL 
	 * in a new tab. Otherwise use system default browser.
	 * 
	 * 
	 * @param aUrl {String}
	 * @returns {void}
	 */
	this.openUrl = function(aUrl)
	{
		const FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";  
		const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";  
		const THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}"; 
		
		let appInfo = Components.classes["@mozilla.org/xre/app-info;1"].
				getService(Components.interfaces.nsIXULAppInfo);  
		
		if(appInfo.ID == FIREFOX_ID || appInfo.ID == SEAMONKEY_ID) {  
			this.openUrlInInternalBrowser(aUrl);
		} else if(appInfo.ID == THUNDERBIRD_ID) {  
			this.openUrlInContentTab(aUrl);
		} else {  
			this.openUrlInDefaultBrowser(aUrl);
		}  
	}; // end openUrl(aUrl)
	
	
	/**
	 * Opens given URL in an internal browser (used if XPCOMViewer is installed 
	 * as an exension on Mozilla Firefox or SeaMonkey).
	 * 
	 * @param aUrl {String}
	 * @returns {void}
	 */
	this.openUrlInInternalBrowser = function(aUrl)
	{
		let wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
				getService(Components.interfaces.nsIWindowMediator);  
		let mainWindow = wm.getMostRecentWindow("navigator:browser");
		
		mainWindow.gBrowser.addTab(aUrl);
	}; // end openUrlInInternalBrowser(aUrl)
	
	
	/**
	 * Opens given URL in a content tab (used if XPCOMViewer is installed 
	 * as an exension on Mozilla Thunderbird).
	 * 
	 * @param aUrl {String}
	 * @returns {void}
	 */
	this.openUrlInContentTab = function(aUrl)
	{
		let tabmail = document.getElementById("tabmail");  
		if(!tabmail) {  
			// Try opening new tabs in an existing 3pane window  
			let mail3PaneWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"].
					getService(Components.interfaces.nsIWindowMediator).
					getMostRecentWindow("mail:3pane");  
			
			if(mail3PaneWindow) {  
				tabmail = mail3PaneWindow.document.getElementById("tabmail");  
				mail3PaneWindow.focus();  
			}  
		}  
			
		if(tabmail)  
			tabmail.openTab("contentTab", {contentPage: aUrl});  
		else  
			window.openDialog(
					"chrome://messenger/content/", "_blank",  
					"chrome,dialog=no,all", null,  
					{ tabType: "contentTab", tabParams: {contentPage: aUrl} }
			); 
	}; // end openUrlInContentTab(aUrl)
	
	
	/**
	 * Opens given URL in a system default browser.
	 * 
	 * @param aUrl {String}
	 * @returns {void}
	 */
	this.openUrlInDefaultBrowser = function(aUrl)
	{
		// first construct an nsIURI object using the ioservice
		let ioservice = Components.classes["@mozilla.org/network/io-service;1"].
				getService(Components.interfaces.nsIIOService);
		let uri = ioservice.newURI(aUrl, null, null);
		let extps = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
				getService(Components.interfaces.nsIExternalProtocolService);
		
		extps.loadURI(uri, null);
	}; // end openUrlInDefaultBrowser(aUrl)
	
}).apply(xpcomviewer.global);