/* Copyright (c) 2010 Ondrej Donek.
 * See the file LICENSE.txt for licensing information. */

// Namespace
if(typeof(xpcomviewer.templates) == "undefined") xpcomviewer.templates = {};
// Implementation
(function() {
	
	/**
	 * Prepare database.
	 * 
	 * @private
	 * @returns {void}
	 */
	this.prepareDb = function()
	{
		let _DB = xpcomviewer.global.getDb();
		
		_DB.executeSimpleSQL(
			"CREATE TABLE IF NOT EXISTS templates (" +
			"  id VARCHAR NOT NULL ," +
			"  lang VARCHAR NOT NULL DEFAULT 'js'," +
			"  source TEXT " + 
			")"
		);
		
		// Try to select template
		let hasTemplate = false;
		let stmt = _DB.createStatement(
				"SELECT * FROM templates WHERE id='class' LIMIT 1 "
		);
		while(stmt.executeStep()) hasTemplate = true;
		
		// Default  code templates
		if(!hasTemplate) {
			// Insert default C++ code templates
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('class','cpp','');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface','cpp','');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_init','cpp','nsresult rv;\nnsCOMPtr<?2> obj = do_CreateInstance(\"?1\", &rv);\nif (NS_FAILED(rv))\n\treturn rv;\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_srv_init','cpp','nsresult rv;\nnsCOMPtr<?2> obj = do_GetService(\"?1\", &rv);\nif (NS_FAILED(rv))\n\treturn rv;\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_const','cpp','?1::?2');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_prop','cpp','');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_method','cpp','');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('result','cpp','?1');"
			);
			
			// Insert default JavaScript code templates
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('class','js','Components.classes[\"?1\"]');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface','js','Components.interfaces.?1');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_init','js','var iface = Components.classes[\"?1\"].\n\t\tgetInstance(Components.interfaces.?2);\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_srv_init','js','var obj = Components.classes[\"?1\"].getService(Components.interfaces.?2);\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_const','js','Components.interfaces.?1.?2');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_prop','js','var foo = iface.?1;\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_method','js','var foo = iface.?1();\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('result','js','Components.results.?1');"
			);
			
			// Insert default Python code templates
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('class','py','components.classes[\"?1\"]');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface','py','components.interfaces.?1');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_init','py','cls = components.classes[\"?1\"]\nabs = cls.createInstance()\niface = cls.queryInterface(?2)\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_srv_init','py','cls = components.classes[\"?1\"]\nsrv = cls.getService(?2)\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_const','py','components.interfaces.?1.?2');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_prop','py','foo = iface.?1\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('iface_method','py','foo = iface.?1();\n');"
			);
			_DB.executeSimpleSQL(
					"INSERT INTO templates VALUES ('result','py','components.results.?1');"
			);
		}
	}; // end prepareDb()
	

	/** 
	 * Returns source of the specified template.
	 * 
	 * @param aName {String}
	 * @param aLang {String}
	 * @returns {String}
	 */
	this.getTemplateSource = function(aName, aLang) 
	{
		let _DB = xpcomviewer.global.getDb();
		
		let stmt = _DB.createStatement(
				"SELECT * " + 
				"FROM templates " + 
				"WHERE id='" + aName + "' AND lang='" + aLang + "' "
		);
		
		while(stmt.executeStep()) { 
			return new String(stmt.row.source); 
		}
		
		return null;
	}; // end getTemplateSource(aName, aLang) 
	

	/** 
	 * Set source of the specified template.
	 * 
	 * @param aName {String}
	 * @param aLang {String}
	 * @param aSource {String}
	 * @returns {void}
	 */
	this.setTemplateSource = function(aName, aLang, aSource) 
	{
		let _DB = xpcomviewer.global.getDb();
		
		_DB.createStatement(
				"UPDATE templates " + 
				"SET source='" + aSource + "' " + 
				"WHERE id='" + aName + "' AND lang='" + aLang + "' "
		).execute();
	}; // end setTemplateSource(aName, aLang, aSource)
	
}).apply(xpcomviewer.templates);
